package edu.umb.cs.umlvm.aspects;

import org.aspectj.lang.Signature;
import org.eclipse.uml2.Interaction;

public aspect DebugSeqToJAST {

    StringBuffer _indent = new StringBuffer();
    
    // pointcuts
    pointcut methods() :
        call( * edu..*.SequenceDiagramToJastMapper.*(..) ) ||
        call( * edu..*.BlockGenerator.*(..) );
    
    pointcut caseinteraction( Interaction interaction ) :
        call( * edu..*.SequenceDiagramToJastMapper.caseInteraction( Interaction ) ) &&
        args( interaction );

    pointcut excludes() :
        call( * edu..*.SequenceDiagramToJastMapper.getCurrentClassName() ) ||
        call( * edu..*.SequenceDiagramToJastMapper.access*(..) ) ||
        call( * edu..*.SequenceDiagramToJastMapper.caseInteraction(..) );

    // advices
    before() :
        methods() && !excludes() {
        printJoinPoint(thisJoinPoint.getSignature());
        indent();
    }

    after() :
        methods() && !excludes() {
        unindent();
    }

    before(Interaction interaction) :
        caseinteraction(interaction) {
        printJoinPoint(thisJoinPoint.getSignature());
        System.out.println( _indent.toString() + ": " + interaction.getName() );
        indent();
    }
    
    after(Interaction interaction) :
        caseinteraction(interaction) {
        unindent();
    }

    
    void printJoinPoint(Signature signature) {
        String msg = _indent.toString() + signature;
        System.out.println( msg.replaceAll( "edu.umb.cs.umlvm.plugins.backend.", "" ) );
    }
    
    void indent() {
        _indent.append( "  " );
    }
    
    void unindent() {
        _indent.deleteCharAt(0);
        _indent.deleteCharAt(0);
    }
}
