package edu.umb.cs.umlvm.aspects;

import java.util.Date;

import edu.umb.cs.umlvm.util.Footprint;
import edu.umb.cs.umlvm.util.Stopwatch;

public aspect MeasurementAspect {

	private long pluginStartTime;
	private long blackboardAccessStartTime;
	private long blackboardAccessTimeInPlugin;
	private long blackboardAccessTimeInTotal;
	private long blackboardRemoteAccessStartTime;
	private long blackboardRemoteAccessTimeInPlugin;
	private long initialMem;
	
	private Stopwatch stopwatch;

	// Definitions of pointcuts
	pointcut executePlugin() :
        execution( * edu.umb.cs.umlvm.plugins.Plugin+.execute() );

	pointcut executeNextPlugin() :
        call( * edu.umb.cs.umlvm.core..executeNextPlugin() );

	// Definitions of advices
	before() : executePlugin() {
		System.out.println( "enter: " + thisJoinPoint.getSignature() );

		initialMem = Footprint.getUsedMemory( true );
		blackboardAccessTimeInPlugin = 0;
		blackboardRemoteAccessTimeInPlugin = 0;
		pluginStartTime = stopwatch.getElapsedTime();
	}

	after() : executePlugin() {
		System.out.println(
			"time : " +
			(stopwatch.getElapsedTime() - pluginStartTime - blackboardAccessTimeInPlugin) );
		System.out.println(	"blackboard (including all): " + blackboardAccessTimeInPlugin );
		System.out.println(	"blackboard (only remote access): " + blackboardRemoteAccessTimeInPlugin );
		System.out.println( "maxmem : "	+ (Footprint.getUsedMemory( false ) - initialMem) );
	}

	after() returning( boolean hasNext ): executeNextPlugin() {

		/*
		 * if( hasNext == false ) { return; } System.out.println( "remmem : " +
		 * (Footprint.getUsedMemory( true ) - initialMem) );
		 */
	}

	// Blackboard
	// Definition of a pointcut
	pointcut blackboardAccess() :
        ( execution( * edu.umb.cs.umlvm.core.blackboard.Blackboard.read*(..) ) ||
          execution( * edu.umb.cs.umlvm.core.blackboard.Blackboard.write*(..) ) ) &&
        this( edu.umb.cs.umlvm.core.blackboard.BlackboardProxy );

	// Definition of an advice
	before() : blackboardAccess(){
		blackboardAccessStartTime = stopwatch.getElapsedTime();
	}

	after() : blackboardAccess(){
		long blackboardAccessTime =
			stopwatch.getElapsedTime() - blackboardAccessStartTime;
		blackboardAccessTimeInPlugin += blackboardAccessTime;
		blackboardAccessTimeInTotal += blackboardAccessTime;
	}

	

	pointcut blackboardRemoteAccess() :
        ( call( * edu.umb.cs.umlvm.core.blackboard.Blackboard.read*(..) ) ||
          call( * edu.umb.cs.umlvm.core.blackboard.Blackboard.write*(..) ) ) &&
		withincode( * edu.umb.cs.umlvm.core.blackboard.BlackboardProxy.*(..) );

	// Definition of an advice
	before() : blackboardRemoteAccess(){
		blackboardRemoteAccessStartTime = stopwatch.getElapsedTime();
	}

	after() : blackboardRemoteAccess(){
		long blackboardAccessTime =
			stopwatch.getElapsedTime() - blackboardRemoteAccessStartTime;
		blackboardRemoteAccessTimeInPlugin += blackboardAccessTime;
	}
	
	

	// Definition of a pointcut
	pointcut main() :
        call( * edu.umb.cs.umlvm.core.Pipeline.start(..) );

	// Definition of an advice
	before() : main(){
		blackboardAccessTimeInTotal = 0;
		stopwatch = new Stopwatch();
		stopwatch.start();
		
		Date d = new Date();
		System.err.println( "==== " + d + " ====" );
	}

	after() : main(){
		System.out.println( "blackboard total time : "
				+ blackboardAccessTimeInTotal );
		System.out.println( "end of measurement" );
	}
}
